package io.hackle.flutter;

import static io.hackle.flutter.Constants.Configs.DEBUG;
import static io.hackle.flutter.Constants.Configs.EXPOSURE_EVENT_DEDUP_INTERVAL_MILLIS;
import static io.hackle.flutter.Constants.Configs.POLLING_INTERVAL_MILLIS;

import androidx.annotation.NonNull;

import java.util.HashMap;
import java.util.Map;

import io.hackle.android.HackleConfig;
import io.hackle.sdk.common.Event;
import io.hackle.sdk.common.ParameterConfig;
import io.hackle.sdk.common.PropertyOperation;
import io.hackle.sdk.common.PropertyOperations;
import io.hackle.sdk.common.User;
import io.hackle.sdk.common.decision.Decision;
import io.hackle.sdk.common.decision.FeatureFlagDecision;

public class ModelConverter {
    User convertHashMapToUser(@NonNull HashMap<String, Object> map) {
        final String id = (String) map.get("id");
        final String deviceId = (String) map.get("deviceId");
        final String userId = (String) map.get("userId");
        final HashMap<String, String> identifiers = (HashMap<String, String>) map.get("identifiers");
        final HashMap<String, Object> properties = (HashMap<String, Object>) map.get("properties");

        final User.Builder builder = User.builder();

        if (id != null) {
            builder.id(id);
        }

        if (deviceId != null) {
            builder.deviceId(deviceId);
        }

        if (userId != null) {
            builder.userId(userId);
        }

        if (identifiers != null) {
            builder.identifiers(identifiers);
        }

        if (properties != null) {
            builder.properties(properties);
        }

        return builder.build();
    }

    HashMap<String, Object> convertUserToHashMap(@NonNull final User user) {
        return new HashMap<String, Object>() {{
            put("id", user.getId());
            put("userId", user.getUserId());
            put("deviceId", user.getDeviceId());
            put("properties", user.getProperties());
            put("identifiers", user.getIdentifiers());
        }};
    }

    HackleConfig convertHashMapToHackleConfig(HashMap<String, Object> map, String wrapperName, String wrapperVersion) {
        final Object exposureEventDedupIntervalMillis = map.get(EXPOSURE_EVENT_DEDUP_INTERVAL_MILLIS);
        final Object pollingIntervalMillis = map.get(POLLING_INTERVAL_MILLIS);
        final Object debug = map.get(DEBUG);

        final HackleConfig.Builder builder = HackleConfig.builder().add("$wrapper_name", wrapperName).add("$wrapper_version", wrapperVersion);

        if (exposureEventDedupIntervalMillis instanceof Integer) {
            builder.exposureEventDedupIntervalMillis((Integer) exposureEventDedupIntervalMillis);
        }

        if (pollingIntervalMillis instanceof Integer) {
            builder.pollingIntervalMillis((Integer) pollingIntervalMillis);
        }

        if (debug instanceof Boolean && (Boolean) debug) {
            builder.eventFlushIntervalMillis(1_000);
            builder.logLevel(3);
        }

        return builder.build();
    }

    Event convertHashMapToEvent(@NonNull HashMap<String, Object> map) {
        final String key = (String) map.get("key");
        final Double value = (Double) map.get("value");
        final HashMap<String, Object> properties = (HashMap<String, Object>) map.get("properties");

        final Event.Builder builder = Event.builder(key);

        if (value != null) {
            builder.value(value);
        }

        if (properties != null) {
            builder.properties(properties);
        }

        return builder.build();
    }

    PropertyOperations convertHashMapToPropertyOperations(@NonNull HashMap<String, Object> map) {
        final HashMap<String, HashMap<String, Object>> converted = new HashMap<>();
        PropertyOperations.Builder builder = PropertyOperations.builder();

        for (Map.Entry<String, Object> entry : map.entrySet()) {
            converted.put(entry.getKey(), (HashMap<String, Object>) entry.getValue());
        }

        for (Map.Entry<String, HashMap<String, Object>> entry : converted.entrySet()) {
            PropertyOperation operation = PropertyOperation.from(entry.getKey());
            HashMap<String, Object> values = entry.getValue();

            switch (operation) {
                case SET:
                    for (Map.Entry<String, Object> innerEntry : values.entrySet()) {
                        builder.set(innerEntry.getKey(), innerEntry.getValue());
                    }
                    break;
                case SET_ONCE:
                    for (Map.Entry<String, Object> innerEntry : values.entrySet()) {
                        builder.setOnce(innerEntry.getKey(), innerEntry.getValue());
                    }
                    break;
                case UNSET:
                    for (Map.Entry<String, Object> innerEntry : values.entrySet()) {
                        builder.unset(innerEntry.getKey());
                    }
                    break;
                case INCREMENT:
                    for (Map.Entry<String, Object> innerEntry : values.entrySet()) {
                        builder.increment(innerEntry.getKey(), innerEntry.getValue());
                    }
                    break;
                case APPEND:
                    for (Map.Entry<String, Object> innerEntry : values.entrySet()) {
                        builder.append(innerEntry.getKey(), innerEntry.getValue());
                    }
                    break;
                case APPEND_ONCE:
                    for (Map.Entry<String, Object> innerEntry : values.entrySet()) {
                        builder.appendOnce(innerEntry.getKey(), innerEntry.getValue());
                    }
                    break;
                case PREPEND:
                    for (Map.Entry<String, Object> innerEntry : values.entrySet()) {
                        builder.prepend(innerEntry.getKey(), innerEntry.getValue());
                    }
                    break;
                case PREPEND_ONCE:
                    for (Map.Entry<String, Object> innerEntry : values.entrySet()) {
                        builder.prependOnce(innerEntry.getKey(), innerEntry.getValue());
                    }
                    break;
                case REMOVE:
                    for (Map.Entry<String, Object> innerEntry : values.entrySet()) {
                        builder.remove(innerEntry.getKey(), innerEntry.getValue());
                    }
                    break;
                case CLEAR_ALL:
                    builder.clearAll();
            }
        }

        return builder.build();
    }


    HashMap<String, Object> convertParameterConfigToHashMap(@NonNull final ParameterConfig config) {
        return new HashMap<String, Object>() {{
            put("parameters", new HashMap<>(config.getParameters()));
        }};
    }

    HashMap<String, Object> convertDecisionToHashMap(@NonNull final Decision decision) {
        return new HashMap<String, Object>() {{
            put("variation", decision.getVariation().toString());
            put("reason", decision.getReason().toString());
            put("config", convertParameterConfigToHashMap(decision.getConfig()));
        }};
    }

    HashMap<String, Object> convertFeatureFlagDecisionToHashMap(@NonNull final FeatureFlagDecision featureFlagDecision) {
        return new HashMap<String, Object>() {{
            put("isOn", featureFlagDecision.isOn());
            put("reason", featureFlagDecision.getReason().toString());
            put("config", convertParameterConfigToHashMap(featureFlagDecision.getConfig()));
        }};
    }
}

